# m-Chat v1.4 by: grymmjack & liquid
# ----------------------------------

uses CFG
uses USER
uses DIR

# m-chat program information constants
const PROGRAM_NAME      = 'm-Chat'
const PROGRAM_VERSION   = 'v1.4'
const PROGRAM_AUTHOR    = 'grymmjack & liquid'
const PROGRAM_AUTHOR_C  = 'grymmjack (grymmjack@home.com)'
const PROGRAM_AUTHOR_A  = 'grymmjack'
const PROGRAM_AUTHOR_T  = 'liquid (liquid@darktech.org)'
const PROGRAM_AUTHOR_I  = 'liquid'
const PROGRAM_CONFIG    = 'MCHAT.CFG'
const PROGRAM_HELP      = 'MCHATH.ANS'
const PROGRAM_ABOUT     = 'MCHATA.ANS'
const PROGRAM_SPLASH    = 'MCHAT.ANS'
const PROGRAM_LOG       = 'MCHAT.LOG'

# variables used for INFORMATION segment of program 
var TOTAL_LOCAL_CHARS   longint
var TOTAL_REMOTE_CHARS  longint
var TOTAL_LOCAL_LINES   longint
var TOTAL_REMOTE_LINES  longint
var TOTAL_COMMANDS_RUN  longint
var START_TIME          longint
var NOW_LOGGING         boolean
var LOG_FILE            string

# constants assigned for better way of referencing configuration data array 
const LOG_HEADER        = 1
const LOG_FOOTER        = 2
const LOG_ALWAYS        = 3
const DEFAULT_SHOW_PATH = 4
const STR_ERROR         = 5
const STR_NORMAL        = 6
const STR_LOCAL_TEXT    = 7
const STR_REMOTE_TEXT   = 8
const STR_ACTION        = 9
const SYSOP_NAME        = 10
const STR_START         = 11
const STR_END           = 12
const ALLOW_USER_CMDS   = 13
const SHOW_LOG_MSGS     = 14
const STR_DARK_COLOR    = 15 
const STR_NORMAL_COLOR  = 16 
const STR_BRIGHT_COLOR  = 17 
const STR_DIVIDER       = 18
const CMD_PREFIX        = 19
const STR_REGULAR_COLOR = 20

# some generic global-scope variables                                       
var counter             integer
var loop                integer
var tmpstr1             string
var tmpstr2             string
var tmplong             longint

# populate a default configuration data array
var CFGDATA string array (1..20)
    # logfile header
    CFGDATA(01) := '+ ++ +++[ |DT @ |TI NOW LOGGING CHAT WITH |CU ]+++ ++ +'
    # logfile footer
    CFGDATA(02) := '- -- ---[ |DT @ |TI CHAT LOGGING WITH |CU ENDED ]--- -- -'
    # always log all chatting
    CFGDATA(03) := '1'
    # default path for showing text files
    CFGDATA(04) := 'c:\mystic\text\'
    # string: command error prefix
    CFGDATA(05) := '|08*|04*|12* |14'
    # string: command normal prefix
    CFGDATA(06) := '|08-|07-|15- |07'
    # string: prefix for local console text
    CFGDATA(07) := '|15'
    # string: prefix for remote text
    CFGDATA(08) := '|07'
    # string: prefix for actions
    CFGDATA(09) := '|10* '
    # sysops name
    CFGDATA(10) := 'sysop'
    # string: being a chat session
    CFGDATA(11) := '|01|09|11 |15' + CFGDATA(SYSOP_NAME) + ' is online|07...|CR|CR'
    # string: end a chat session
    CFGDATA(12) := '|CR|CR|01|09|11 |15' + CFGDATA(SYSOP_NAME) + ' is offline|07...|CR|CR|PA'
    # allow users to issue commands?
    CFGDATA(13) := '0'
    # show log file messages?
    CFGDATA(14) := '1'
    # string: dark color
    CFGDATA(15) := '|01'
    # string: normal color
    CFGDATA(16) := '|09'
    # string: bright color
    CFGDATA(17) := '|15'
    # string: text divider
    CFGDATA(18) := '|16|01- -- ---|09-|01-|09--|01-|09---|01-|09|$D45-|01-|09---|01-|09--|01-|09-|01--- -- -' 
    # log prefix/postfix for command entries
    CFGDATA(19) := '*'
    # string: regular color
    CFGDATA(20) := '|07'






# -------------------------------------------------------------------
# functions and procedures
# -------------------------------------------------------------------

# clear the screen in a non-ugly fashion ;)
procedure clearscreen
begin
  if NOW_LOGGING = true then
    fwriteln(1, CFGDATA(CMD_PREFIX) + ' CLEARED THE SCREEN ' + CFGDATA(CMD_PREFIX))
  endif

  write('|16|CL')
pend



# parse mci codes -- this function was made to evaluate date, time, and user
# for inclusion in log headers and footers. it also makes the formatting of
# internal displays much easier on me the programmer ;)
function parse_mci(originalString string) : string
begin
  # m-Chat MCI codes:
  # -----------------
  # |DC = Dark color
  # |NC = Normal color
  # |BC = Bright color
  # |RC = Regular color
  # |TI = Time
  # |DT = Date
  # |CU = Current user
  #
  # PLUS ALL MYSTIC CODES!@$@

  # control variable that lets us know when we're done
  var allParsed boolean
  # the modified/parsed string
  var parsedString string
  # the current character we are looking at in the string
  var posCurChar integer
  
  # reset our position to 0 and our state to unparsed
  posCurChar := 0
  allParsed := false

  repeat
    # incrmement the position counter
    posCurChar := posCurChar + 1

    # test for an mci
    if copy(originalString, posCurChar, 1) = '|' then
      # do m-chat specific mcis here
      if upper(copy(originalString, posCurChar, 3)) = '|DC' then
        # append the parsed output with our interpreted mci values
        parsedString := parsedString + CFGDATA(STR_DARK_COLOR)
        # increment the position counter so that the mci is not evaluated further
        posCurChar := posCurChar + 2
      elseif upper(copy(originalString, posCurChar, 3)) = '|NC' then
        parsedString := parsedString + CFGDATA(STR_NORMAL_COLOR)
        posCurChar := posCurChar + 2
      elseif upper(copy(originalString, posCurChar, 3)) = '|BC' then
        parsedString := parsedString + CFGDATA(STR_BRIGHT_COLOR)
        posCurChar := posCurChar + 2
      elseif upper(copy(originalString, posCurChar, 3)) = '|RC' then
        parsedString := parsedString + CFGDATA(STR_REGULAR_COLOR)
        posCurChar := posCurChar + 2
      elseif upper(copy(originalString, posCurChar, 3)) = '|TI' then
        parsedString := parsedString + timestr(datetime, 1)
        posCurChar := posCurChar + 2
      elseif upper(copy(originalString, posCurChar, 3)) = '|DT' then
        parsedString := parsedString + datestr(datetime, 1)
        posCurChar := posCurChar + 2
      elseif upper(copy(originalString, posCurChar, 3)) = '|CU' then
        parsedString := parsedString + useralias
        posCurChar := posCurChar + 2
      else
        # otherwise just append a pipe to the parsed output thus
        # allowing all mystic mcis as well
        parsedString := parsedString + '|'
      endif
    else
      # if its not an mci, just append the character to the parsed string
      parsedString := parsedString + copy(originalString, posCurChar, 1)
    endif

    # if our position is at the end of the string or greater
    if posCurChar >= length(originalString) then
      # exit the function and return the parsed string
      allParsed := true
      parse_mci := parsedString
    endif
    
  # repeat this sequence until we're all done =]  
  until allParsed = true

pend



# this procedure creates the log file and keeps it open during chat
procedure create_log(args string)
begin
  # if the argument supplied exists, append to the file
  if fexist(args) then
    fopen(1, text, append, args)
    LOG_FILE := args
  # otherwise create it from scratch
  else 
    fopen(1, text, rewrite, args)
    LOG_FILE := args
  endif

  # write the log header to the file
  fwriteln(1, parse_mci(CFGDATA(LOG_HEADER)))
  fwriteln(1, '')

  # if log messages are enabled then show that we started logging
  if CFGDATA(SHOW_LOG_MSGS) = '1' then
    writeln('|CR' + CFGDATA(STR_NORMAL) + LOG_FILE + ' logfile started...')
  endif
pend



# this procedure closes the log file and stops logging
procedure close_log
begin
  # write the log footer to the file
  fwriteln(1, '')
  fwriteln(1, parse_mci(CFGDATA(LOG_FOOTER)))
  fwriteln(1, '')
  fwriteln(1, '')
  
  # close the file
  fclose(1)
  
  # if log messages are enabled then show that we closed the log
  if CFGDATA(SHOW_LOG_MSGS) = '1' then
    writeln('|CR' + CFGDATA(STR_NORMAL) + LOG_FILE + ' logfile closed...')
  endif
pend



# this procedure writes to the log file
procedure write_log(args string)
begin
  # first check to see if we're logging because this routine gets called
  # regardless of this fact
  if NOW_LOGGING = true then
    fwriteln(1, args)
  endif
pend



# this procedure is the handler for the /log command it calls upon
# create_log and close_log accordingly..
procedure log(args string)
begin
  # if no argument is given...
  if args = '' then
    # and we are already logging
    if NOW_LOGGING = true then
      # then close the log
      NOW_LOGGING := false
      close_log
    # otherwise, display an error and prompt for argument
    else
      writeln('|CR' + CFGDATA(STR_ERROR) + 'log to what file!?')
    endif
    exit
  endif

  # if an argument is supplied
  # check to see if it has a .log extension
  if upper(copy(args, length(args)-3, 4)) = '.LOG'
    # if so, lets start logging with that logfile
    NOW_LOGGING := true
    if pos(pathchar, args) = 0 then
      create_log(cfglogspath + args)
    else
      create_log(args)
    endif
  else
    # otherwise lets give the argument an extension
    # and then start logging
    NOW_LOGGING := true
    if pos(pathchar, args) = 0 then
      create_log(cfglogspath + args + '.LOG')
    else
      create_log(args)
    endif
  endif

pend



# read configuration data from MCHAT.CFG into CFGDATA() array
procedure read_config
begin

  if fexist(cfgdatapath + PROGRAM_CONFIG) then

    counter := 0
    fopen(1, text, reset, cfgdatapath + PROGRAM_CONFIG)
      while not eof(1)
        freadln(1, tmpstr1)
        if copy(tmpstr1, 1, 1) <> '#' then
          if copy(tmpstr1, 1, 1) <> '' then
            if copy(tmpstr1, 1, 1) <> ' ' then
              counter := counter + 1
              CFGDATA(counter) := tmpstr1
            endif
          endif
        endif
      wend
    fclose(1)

  else

    # if configuration file does not exist, create it using the defaults
    dispfile(cfgtextpath + PROGRAM_SPLASH)
    writeln(CFGDATA(STR_ERROR) + cfgdatapath + PROGRAM_CONFIG + ' does NOT exist.')
    writeln(CFGDATA(STR_ERROR) + 'creating this file for you. edit it with a text editor')
    writeln(CFGDATA(STR_ERROR) + 'to customize the way ' + PROGRAM_NAME + ' works..')
    fopen(1, text, rewrite, cfgdatapath + 'mchat.cfg')
      fwriteln(1, '# ' + PROGRAM_CONFIG + ' -- configuration file for ' + PROGRAM_NAME)
      fwriteln(1, '# NOTE: all lines that begin with a "#" are comments...')
      fwriteln(1, '#       all mci codes are valid as are color codes!')
      fwriteln(1, '#')
      fwriteln(1, '')
      fwriteln(1, '# logfile header:')
      fwriteln(1, CFGDATA(LOG_HEADER))
      fwriteln(1, '# logfile footer:')
      fwriteln(1, CFGDATA(LOG_FOOTER))
      fwriteln(1, '# always log chat? 0 = no, 1 = yes (to logpath/mchat.log)')
      fwriteln(1, CFGDATA(LOG_ALWAYS))
      fwriteln(1, '# default path for showing text files:')
      fwriteln(1, CFGDATA(DEFAULT_SHOW_PATH))
      fwriteln(1, '# string: command error prefix')
      fwriteln(1, CFGDATA(STR_ERROR))
      fwriteln(1, '# string: command success prefix')
      fwriteln(1, CFGDATA(STR_NORMAL))
      fwriteln(1, '# string: text entered by local console (color code like |15 works good!):')
      fwriteln(1, CFGDATA(STR_LOCAL_TEXT))
      fwriteln(1, '# string: text entered by remote user (color code like |07 works good!):')
      fwriteln(1, CFGDATA(STR_REMOTE_TEXT))
      fwriteln(1, '# string: action prefix (something like |10*  works good!):')
      fwriteln(1, CFGDATA(STR_ACTION))
      fwriteln(1, '# string: sysops name')
      fwriteln(1, CFGDATA(SYSOP_NAME))
      fwriteln(1, '# string: beginning a chat session')
      fwriteln(1, CFGDATA(STR_START))
      fwriteln(1, '# string: end a chat session')
      fwriteln(1, CFGDATA(STR_END))
      fwriteln(1, '# allow remote users to issue commands? 0 = no, 1 = yes')
      fwriteln(1, CFGDATA(ALLOW_USER_CMDS))
      fwriteln(1, '# show logfile messages? 0 = no, 1 = yes')
      fwriteln(1, CFGDATA(SHOW_LOG_MSGS))
      fwriteln(1, '# string: dark color (something like |08 works great!):')
      fwriteln(1, CFGDATA(STR_DARK_COLOR))
      fwriteln(1, '# string: normal color (something like |07 works great!):')
      fwriteln(1, CFGDATA(STR_NORMAL_COLOR))
      fwriteln(1, '# string: bright color (something like |15 works good!):')
      fwriteln(1, CFGDATA(STR_BRIGHT_COLOR))
      fwriteln(1, '# string: divider (remember all mystic mcis work!):')
      fwriteln(1, CFGDATA(STR_DIVIDER))
      fwriteln(1, '# log prefix/postfix for command entries:')
      fwriteln(1, CFGDATA(CMD_PREFIX))
      fwriteln(1, '# string: regular color (something like |07 works good!):')
      fwriteln(1, CFGDATA(STR_REGULAR_COLOR))
      
    fclose(1)
    writeln(CFGDATA(STR_NORMAL) + cfgdatapath + PROGRAM_CONFIG + ' created...|CR|PA')

  endif

pend



# display m-chat command help
procedure showhelp
begin
  # if logging is on, make a note that we showed the help screen in the log
  write_log(CFGDATA(CMD_PREFIX) + ' SHOWED MCHAT HELP ' + CFGDATA(CMD_PREFIX))
  
  # if help screen exists, display it
  if fexist(cfgtextpath + PROGRAM_HELP) then
    dispfile(cfgtextpath + PROGRAM_HELP)
  else
    # otherwise show a stock built in help screen.
    writeln('')
    writeln(CFGDATA(STR_DIVIDER))
    writeln(parse_mci('|BC' +  padct(PROGRAM_NAME + '|RC ' + PROGRAM_VERSION + ' |NCCommand Help', 80, ' ')))
    writeln(CFGDATA(STR_DIVIDER))
    writeln(parse_mci('|NC/|BCA|DC, |NC/|BCABOUT|DC......: |RCShow m-Chat specific stuff, like credits, etc.'))
    writeln(parse_mci('|NC/|BCC|DC, |NC/|BCCLS|DC........: |RCClear the screen of all text.'))
    writeln(parse_mci('|NC/|BC?|DC, |NC/|BCH|DC, |NC/|BCHELP|DC...: |RCShow m-Chat command help.'))
    writeln(parse_mci('|NC/|BCI|DC, |NC/|BCINFO|DC.......: |RCShow m-Chat information.'))
    writeln(parse_mci('|NC/|BCL|DC, |NC/|BCLOG|DC........: |RCSave input to <logfile>, or stop logging if already on.'))
    writeln(parse_mci('|NC/|BCM|DC, |NC/|BCMSG|DC........: |RCSend a multinode message (/msg # <text>) where # = node #.'))
    writeln(parse_mci('|NC |BC |DC  |NC/|BCME|DC.........: |RCPerform an action (like IRC).'))
    writeln(parse_mci('|NC/|BCT|DC, |NC/|BCTYPE|DC.......: |RCAttempt to display <text file>.'))
    writeln(parse_mci('|NC/|BCW|DC, |NC/|BCWHO|DC........: |RCShow whos online.'))
    writeln(parse_mci('|NC |BC |DC  |NC^|BCY|DC..........: |RCYank current line of text.'))
    writeln(CFGDATA(STR_DIVIDER))
  endif
pend



# display m-chat information
procedure showinfo
begin
  # if logging is on, make a note that we showed the information in the log
  write_log(CFGDATA(CMD_PREFIX) + ' SHOWED MCHAT SESSION INFO ' + CFGDATA(CMD_PREFIX))
  
  # display the information
  clearscreen
  writeln(parse_mci('|BC' +  padct(PROGRAM_NAME + '|RC ' + PROGRAM_VERSION + ' |NCInformation', 83, ' ')))
  writeln(CFGDATA(STR_DIVIDER))
  writeln(parse_mci('|BCStarted on|DC.......: |NC' + datestr(START_TIME, 1) + ' @ ' + timestr(START_TIME, 1)))
    write(parse_mci('|BCLogging|DC..........: |NC'))
  if NOW_LOGGING = true then
    writeln(parse_mci('ON |DC(|BC' + LOG_FILE + '|DC)|NC ' + int2str(filesize(1)) + ' bytes|CR'))
  else
    writeln('OFF|CR')
  endif
  writeln(parse_mci('|NC[|BCTotals for this chat|NC]|DC---------- --- --  -'))
  writeln(parse_mci('|DC |NC' + CFGDATA(SYSOP_NAME) + '|DC '))
  writeln(parse_mci('|BCCharacters Typed|DC.: |NC' + int2str(TOTAL_LOCAL_CHARS)))
  writeln(parse_mci('|BCLines Entered|DC....: |NC' + int2str(TOTAL_LOCAL_LINES) + '|CR'))
  writeln(parse_mci('|DC |NC' + useralias + '|DC '))
  writeln(parse_mci('|BCCharacters Typed|DC.: |NC' + int2str(TOTAL_REMOTE_CHARS)))
  writeln(parse_mci('|BCLines Entered|DC....: |NC' + int2str(TOTAL_REMOTE_LINES) + '|CR'))
    write(parse_mci('|BCDifference of|DC....: |NC'))
  if TOTAL_LOCAL_CHARS + TOTAL_LOCAL_LINES > TOTAL_REMOTE_CHARS + TOTAL_REMOTE_LINES then
    tmplong := TOTAL_LOCAL_CHARS + TOTAL_LOCAL_LINES - TOTAL_REMOTE_CHARS - TOTAL_REMOTE_LINES
    writeln(int2str(tmplong) + parse_mci(' |DC(|BCin ' + CFGDATA(SYSOP_NAME) + 's favor|DC)'))
  else
    tmplong := TOTAL_REMOTE_CHARS + TOTAL_REMOTE_LINES - TOTAL_LOCAL_CHARS - TOTAL_REMOTE_LINES
    writeln(int2str(tmplong) + parse_mci(' |DC(|BCin ' + useralias + 's favor|DC)|CR'))
  endif
  writeln(parse_mci('|BCCommands Issued|DC..: |NC' + int2str(TOTAL_COMMANDS_RUN) + '|CR'))
  writeln(parse_mci('|NC[|BC' + PROGRAM_NAME + ' Specific Information|NC]|DC---------- --- ---  -'))
  writeln(parse_mci('|BCVersion|DC..........: |NC' + PROGRAM_VERSION))
    write(parse_mci('|BC' + padrt(PROGRAM_SPLASH + '|DC', 20, '.') + ': |NC'))
  if fexist(cfgtextpath + PROGRAM_SPLASH) then
    writeln('FOUND! (' + cfgtextpath + PROGRAM_SPLASH + ')')
  else
    writeln('MISSING!')
  endif
    write(parse_mci('|BC' + padrt(PROGRAM_ABOUT + '|DC', 20, '.') + ': |NC'))
  if fexist(cfgtextpath + PROGRAM_ABOUT) then
    writeln('FOUND! (' + cfgtextpath + PROGRAM_ABOUT + ')')
  else
    writeln('MISSING!')
  endif
    write(parse_mci('|BC' + padrt(PROGRAM_HELP + '|DC', 20, '.') + ': |NC'))
  if fexist(cfgtextpath + PROGRAM_HELP) then
    writeln('FOUND! (' + cfgtextpath + PROGRAM_HELP + ')')
  else
    writeln('MISSING!')
  endif
  writeln(CFGDATA(STR_DIVIDER))
pend



# display m-chat about, credits, etc. 
procedure showabout
begin
  # if logging is on, make a note that we showed the about in the log
  write_log(CFGDATA(CMD_PREFIX) + ' SHOWED MCHAT ABOUT SCREEN ' + CFGDATA(CMD_PREFIX))
 
  # if about screen exists, display it
  if fexist(cfgtextpath + PROGRAM_ABOUT) then
    dispfile(cfgtextpath + PROGRAM_ABOUT)
  else
    # otherwise show a built in stock version.
    clearscreen
    writeln(CFGDATA(STR_DIVIDER))
    writeln(parse_mci(padct('|NCAbout |BC' + PROGRAM_NAME + '|RC ' + PROGRAM_VERSION, 86, ' ')))
    writeln('')
    writeln(parse_mci('|NC(|BCCredits|NC)|DC---------- --- --  -'))
    writeln(parse_mci('|BCCode|DC......: |NC' + PROGRAM_AUTHOR_C))
    writeln(parse_mci('|BCArtwork|DC...: |NC' + PROGRAM_AUTHOR_A))
    writeln(parse_mci('|BCTesting|DC...: |NC' + PROGRAM_AUTHOR_T))
    writeln(parse_mci('|BCIdeas|DC.....: |NC' + PROGRAM_AUTHOR_I + '|CR'))
    writeln(parse_mci('|NC(|BCWhy was it made?|NC)|DC---------- --- --  -'))
    writeln(parse_mci('|RCm-Chat was written in 100% MPL. It was made because Mystic BBSs default single-'))
    writeln(parse_mci('line chat mode leaves alot to be desired. Hopefully, m-Chat can fill the void'))
    writeln(parse_mci('and make Mystic BBS chat a little bit more fun for everyone.|CR'))
    writeln(parse_mci('|NC(|BCMore to Come...|NC)|DC---------- --- --  -'))
    writeln(parse_mci('|RCIf you enjoy m-Chat, be sure to connect to the following boards to get more'))
    writeln(parse_mci('good mods. Thanks for using m-Chat!|CR'))
    writeln(parse_mci('|NC(|BCOfficial m-Chat Support BBSs|NC)|DC---------- --- --  -'))
    writeln(parse_mci('|BCsector7 BBS|DC.....: |NCsector7bbs.com |DC- |RCsysop|DC: |RCgrymmjack'))
    writeln(parse_mci('|BCthe dominion BBS|DC: |NCthedominion.darktech.org |DC - |RCsysop|DC: |RCliquid'))
    writeln('')
    writeln(CFGDATA(STR_DIVIDER))
  endif
pend



procedure nodemsg(args string)
begin
  # make two variables for working with one for node num one for msg
  var node string
  var message string

  # make sure we have an argument if not error and exit 
  if args = '' then
    writeln('|CR' + CFGDATA(STR_ERROR) + 'proper syntax: /msg # <text> (ie: /msg 1 hey liquid!)')
    exit
  endif

  # get the node number
  node := copy(args, 1, pos(' ', args) - 1)
  # get the message
  message := copy(args, pos(node, args) + 2, length(args))

  # setup our menu command argument with gathered data
  tmpstr1 := node + ';' + message

  # run the menu command - send the message
  menucmd('NS', tmpstr1)
  
  # log the fact that we did this
  write_log(CFGDATA(CMD_PREFIX) + ' --- MULTINODE MESSAGE TO NODE: ' + node + ' --- ' + CFGDATA(CMD_PREFIX))
  write_log(CFGDATA(CMD_PREFIX) + ' --- MESSAGE: ' + message + ' ' + CFGDATA(CMD_PREFIX))

  # add a linefeed
  writeln(parse_mci('|CR' + CFGDATA(STR_NORMAL) + 'multinode message sent to node |BC' + node + '|NC.|DC.'))
pend



# actually show the file for the /type command
procedure typefile(args string)
begin
  # make sure we're starting on a new line
  write('|CR')
  # display the file
  dispfile(args)
  # write to the log that we showed the file
  write_log(CFGDATA(CMD_PREFIX) + ' TYPED ' + args +  ' ' + CFGDATA(CMD_PREFIX))
pend



# attempt to display a text file calls on typefile
procedure showfile(args string)
begin
  
  var mysticdat string
  var usersdat string

  # set the forbidden files up
  mysticdat := cfgsyspath + 'MYSTIC.DAT'
  usersdat := cfgdatapath + 'USERS.DAT'
  
  # if argument is not given, prompt for it
  if args = '' then
    writeln('|CR' + CFGDATA(STR_ERROR) + 'usage: /TYPE <file (ie: C:\MYSTIC\TEXT\WELCOME.ANS)>')
    exit
  endif

  # get the file extension if there is one 
  if copy(args, length(args) - 3, 1) = '.' then
    var ext string
    ext := upper(copy(args, length(args) - 3, 4))
  endif
 
  # prevent MYSTIC.DAT, USER.DAT, *.EXE and *.OVR from being displayed at all!
  if upper(args) = upper(mysticdat) or upper(args) = upper(usersdat) or upper(ext) = '.EXE' or upper(ext) = '.COM' or upper(ext) = '.OVR' then
    writeln('|CR' + CFGDATA(STR_ERROR) + 'you may NOT view ' + args + '. it is a security risk.')
    exit
  endif

  # if a default show path exists in the config, try to find the file there
  if CFGDATA(DEFAULT_SHOW_PATH) <> '' then
    if fexist(CFGDATA(DEFAULT_SHOW_PATH) + args) = true then
      typefile(CFGDATA(DEFAULT_SHOW_PATH) + args)
    endif
  endif
  
  # check to see if the file exists, if it does, display it
  if fexist(args) then
    typefile(args)
  # if all else fails, check the default text directory for the file
  elseif fexist(cfgtextpath + args) then
    typefile(cfgtextpath + args)
  # check to see if an extension was ommited
  elseif fexist(cfgtextpath + args + '.ANS') then
    typefile(cfgtextpath + args + '.ANS')
  elseif fexist(cfgtextpath + args + '.ASC') then
    typefile(cfgtextpath + args + '.ASC')
  else    
    # if we can't find the file after all that stuff, show an error
    writeln('|CR' + CFGDATA(STR_ERROR) + 'file not found: ' + args)
  endif

pend






# -------------------------------------------------------------------
# main program
# -------------------------------------------------------------------
begin
  # get some information from mystic about the bbs
  getthisuser
  getcfg

  # populate the CFGDATA() array with values from the MCHAT.CFG file
  # if it exists, otherwise make it from scratch.
  read_config
  
  # our control variable which determines the program flow
  var done boolean
  # the character pressed by either remote or local end
  var intext char
  # the current line of text
  var inline string
  # a temporary variable for use in determining line stuff
  var tmpline string
  # variable for condition testing whether or not to draw the character
  var drawText boolean
  # variable for condition testing whether to append intext to inline
  var addTextToLine boolean
  # variable to determine if we should add line to log
  var addToLog boolean
  # variables for word-wrap code
  var wordCounter integer
  var theWord integer
  var foundLastWord boolean
  
  # reset the session counter variables
  TOTAL_LOCAL_CHARS := 0
  TOTAL_REMOTE_CHARS := 0
  TOTAL_LOCAL_LINES := 0
  TOTAL_REMOTE_LINES := 0
  
  # turn off logging by default and unset the LOG_FILE variable
  NOW_LOGGING := false
  LOG_FILE := ''

  # push the current date and time into our session start time
  START_TIME := datetime

  # if always logging is enabled, then create/append to the MCHAT.LOG file
  # in the BBS log directory
  if CFGDATA(LOG_ALWAYS) = '1' then
    log(PROGRAM_LOG)
  endif   
  
  # show splash ansi if it exists
  dispfile(cfgtextpath + PROGRAM_SPLASH)

  # display the "begin a chat session" string
  write(CFGDATA(STR_START))

  # set our control variables
  done := false

  # this is where the program begins reading characters from both sides
  # until we actually quit the program by typing /q or /quit ;)
  repeat
    # do some variable stuff that initialize things to the right state
    drawText := true
    addTextToLine := false
    addToLog := true
    intext := readkey
    theWord := 0
    wordCounter := 0
    foundLastWord := false

    # if the line does not contain ANY spaces lets just log that
    # line of input and reset the line to nothing
    if wherex = 80 then
      if pos(' ', inline) = 0 then
        write_log(inline)
        inline := ''
      endif
    endif

    # word-wrap code triggers when we hit column 79 (column 80
    # caused strange problems because of the linefeed being automatically
    # added when the cursor reaches column 80) so we do our stuff on 79
    if wherex >= 79 then
      # set the wordcounter beginning position to be length of our
      # input line + 1 (necessary for the way the program goes) 
      wordCounter := length(inline) + 1
      repeat
        # decrement the word counter (because this is first in the
        # repeat until loop, we had to add a +1 to the length of
        # the input line before entering the loop)
        wordCounter := wordCounter - 1
        # set the loop variable to equal the position of a space if it's found
        loop := pos(' ', copy(inline, wordCounter, 1))
        # if we found a space, that's good enough, we started from the end of
        # the input line, so the first space means we've found the first word.
        if loop = 1 then 
          # so lets exit the repeat until loop
          foundLastWord := true
        endif
      # continue searching for the last word on the line until we either
      # run out of characters to check or we actually find one
      until foundLastWord = true or wordCounter = 0

      # if we found a word and the line contained a space...
      if wordCounter > 0 and pos(' ', inline) > 0 then
        # set theWord to equal the position of the last space on the line
        theWord := wordCounter
        # and set tmpstr1 to be the text UP TO the last space in the line
        tmpstr1 := copy(inline, 1, theWord)
        # and set tmpstr2 to be the text AFTER the last space on the line
        tmpstr2 := copy(inline, theWord, length(inline))

        # if the sysop is typing, lets let the text wrap for him
        if islocalkey then
          # wrap the text for sysop
          write(CFGDATA(STR_LOCAL_TEXT) + '|[X' + int2str(theWord) + strrep(' ', length(tmpstr2)) + '|CR' + copy(tmpstr2, 2, length(tmpstr2)))
          # increment total line counter for local sysop
          TOTAL_LOCAL_LINES := TOTAL_LOCAL_LINES + 1
        # otherwise, lets wrap the text for the user
        else
          # wrap the text for user
          write(CFGDATA(STR_REMOTE_TEXT) + '|[X' + int2str(theWord) + strrep(' ', length(tmpstr2)) + '|CR' + copy(tmpstr2, 2, length(tmpstr2)))
          # increment total line counter for remote user
          TOTAL_REMOTE_LINES := TOTAL_REMOTE_LINES + 1
        endif

        # set the line of input to be the wrapped text UP TO where we did
        # the wrapping
        inline := copy(inline, 1, theWord)
        # if the first character in this line is a space, skip it when
        # we add to the log
        if copy(inline, 1, 1) = ' ' then
          write_log(copy(inline, 2, length(inline)))
        # otherwise just write the stuff to the log
        else
          write_log(inline)
        endif
        # and lastly set the new input line to equal the wrapped text
        inline := tmpstr2
      endif

    endif

    # backspace
    if intext = chr(8) then
      # draw the backspace (by default it is non-destructive)
      write('|[D01 ')
      # modify the input line to remove the last character typed.
      inline := copy(inline, 1, length(inline) - 1)
    endif

    # if is movement/navigation key
    if isarrow = true then
      # home key
      if intext = chr(71) then
        write('|[X01|$D79 |[X01')
        # for the sake of simplicity home is destructive
        inline := ''
      # up arrow key
      elseif intext = chr(72) then
        # do nothing
        drawText := false
      # left arrow
      elseif intext = chr(75) then
        # draw the arrow (by default it is non-destructive)
        write('|[D01 ')
        # modify the input line to remove the last character typed.
        inline := copy(inline, 1, length(inline) - 1)
      # delete
      elseif intext = chr(83) then
        # draw the delete (by default it does nothing)
        write('|[D01 ')
        # modify the input line to remove the last character typed.
        inline := copy(inline, 1, length(inline) - 1)
      # right arrow
      elseif intext = chr(77) then
        # draw the right arrow
        write('|[C01')
        intext := ' '
        drawText := true
      # end
      elseif intext = chr(79) then
        # do nothing
        drawText := false
      # down arrow
      elseif intext = chr(80) then
        # do nothing
        drawText := false
      endif
      # ctrl-y (yank)
      elseif intext = chr(25) then
        # yank the line
        write('|[X01|$D79 |[X01')
        #remove the line of input
        inline := ''
      endif
    endif

    # action handler like irc
    if upper(copy(inline, 1, 3)) = '/ME' and intext = chr(13) then
      # if line contains more than just "/me"
      if length(inline) > 3 then
        # if it's sysop doing action then make name show as sysops
        if islocalkey then
          write('|[X01' + CFGDATA(STR_ACTION) + CFGDATA(SYSOP_NAME) + copy(inline, 4, length(inline)))
          write_log(stripmci(CFGDATA(STR_ACTION)) + CFGDATA(SYSOP_NAME) + copy(inline, 4, length(inline)))
          # increment total lines typed for sysop
          TOTAL_LOCAL_LINES := TOTAL_LOCAL_LINES + 1
        # otherwise make the action for the user
        else
          write('|[X01' + CFGDATA(STR_ACTION) + useralias + copy(inline, 4, length(inline)))
          write_log(stripmci(CFGDATA(STR_ACTION)) + useralias + copy(inline, 4, length(inline)))
          # increment total lines typed for remote user
          TOTAL_REMOTE_LINES := TOTAL_REMOTE_LINES + 1
        endif

        # unset inline since it's a new line
        inline := ''
        addToLog := false
        drawText := true
      # otherwise just log the line
      else
        addToLog := true
        drawText := false
      endif
    endif
    
    # command handler
    if copy(inline, 1, 1) = '/' and intext = chr(13) then
      # check to make sure the command follows security config
      if CFGDATA(ALLOW_USER_CMDS) = '1' or islocalkey then
        # modify the total commands run count 
        TOTAL_COMMANDS_RUN := TOTAL_COMMANDS_RUN + 1
        # about command
        if upper(copy(inline, 2, 5)) = 'ABOUT' or upper(copy(inline, 2, 1)) = 'A' then
          showabout
        # clearscreen command
        elseif upper(copy(inline, 2, 3)) = 'CLS' or upper(copy(inline, 2, 1)) = 'C' then
          clearscreen
        # help command
        elseif upper(copy(inline, 2, 4)) = 'HELP' or upper(copy(inline, 2, 1)) = 'H' or upper(copy(inline, 2, 1)) = '?' then
          showhelp
        # information command
        elseif upper(copy(inline, 2, 4)) = 'INFO' or upper(copy(inline, 2, 1)) = 'I' then
          showinfo
        # log command
        elseif upper(copy(inline, 2, 3)) = 'LOG' or upper(copy(inline, 2, 1)) = 'L' then
          # if line contains an argument run the procedure with it
          if pos(' ', inline) > 0 then
            log(copy(inline, pos(' ', inline) + 1, length(inline)))
          else
            # otherwise run with no args and cause the error message to show up
            log('')
          endif
        # multinode msg command
        elseif upper(copy(inline, 2, 3)) = 'MSG' or upper(copy(inline, 2, 1)) = 'M' then
          if pos(' ', inline) > 0 then
            nodemsg(copy(inline, pos(' ', inline) + 1, length(inline)))
          else
            nodemsg('')
          endif
        # quit command
        elseif upper(copy(inline, 2, 4)) = 'QUIT' or upper(copy(inline, 2, 1)) = 'Q' then
          done := true
        # type command
        elseif upper(copy(inline, 2, 4)) = 'TYPE' or upper(copy(inline, 2, 1)) = 'T' then
          # if line contains an argument run the procedure with it
          if pos(' ', inline) > 0 then
            showfile(copy(inline, pos(' ', inline) + 1, length(inline)))
          else
            # otherwise call procedure with no argument resulting in an error
            showfile('')
          endif
        # who's online command
        elseif upper(copy(inline, 2, 3)) = 'WHO' or upper(copy(inline, 2, 1)) = 'W' then
          clearscreen
          menucmd('NW','')
        else
          # if it's starting with a slash but doesn't have a valid command then
          # draw a linefeed
          addToLog := true
          write('|CR')
          drawText := false
          # if the first character in this line is a space, skip it when
          # we add to the log
          if addToLog = true then
            if copy(inline, 1, 1) = ' ' then
              write_log(copy(inline, 2, length(inline)))
            # otherwise just write the stuff to the log
            else
              write_log(inline)
            endif
          endif
        endif
        # clear the input line
        inline := ''
        drawText := false
        addToLog := false
      endif
    endif

    # carriage return handler
    if intext = chr(13) then
      if addToLog = true then
        if copy(inline, 1, 1) = ' ' then
          write_log(copy(inline, 2, length(inline)))
        else
          write_log(inline)
        endif
      endif
        
      if islocalkey then
        TOTAL_LOCAL_LINES := TOTAL_LOCAL_LINES + 1
      else
        TOTAL_REMOTE_LINES := TOTAL_REMOTE_LINES + 1
      endif
      
      if drawText = true then
        write('|CR')
      endif

      inline := ''

    # all other keys handler
    else
      addTextToLine := true
      drawText := true

      # do not add text to the line if it was a backspace
      if intext = chr(8) then
        addTextToLine := false
      endif
      
      # do not draw anything if it was an invisible special char.
      if isarrow or intext = chr(25) then
        drawText := false
        addTextToLine := false
      endif

      if intext = chr(13) then
        drawText := false
      endif
      
      # if it's sysop pressing the key
      if islocalkey then
        # and the drawText flag is set
        if drawText = true then
          # draw the character on screen
          write(CFGDATA(STR_LOCAL_TEXT) + intext)
        endif
        # increment sysop character counter
        TOTAL_LOCAL_CHARS := TOTAL_LOCAL_CHARS + 1
      else
        # otherwise it is a remote key being pressed
        # and if the drawText flag is set
        if drawText = true then
          # draw the text
          write(CFGDATA(STR_REMOTE_TEXT) + intext)
        endif
        # increment remote character counter
        TOTAL_REMOTE_CHARS := TOTAL_REMOTE_CHARS + 1
      endif
      
      # append the character to our line variable
      # if the addText flag is set
      if addTextToLine = true then
        inline := inline + intext
      endif
    endif

  # repeat this sequence until we quit
  until done = true

  # show the 'chat ended' string once we quit
  write(CFGDATA(STR_END)) 

  # and if we are logging, close the logfile
  if NOW_LOGGING = true then
    close_log
  endif

  # return back to the bbs after calling with ! sysop macro
  stuffkey(chr(13))

end

